/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbvtkMIPCreator.h,v $
  Language:  C++
  Date:      $Date: 2009/05/15 14:58:01 $
  Version:   $Revision: 1.8 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 * \brief Short description in one line
 * 
 * Long description which 
 * can span multiple lines
 */
/**
 * \file 
 * \brief Pattern for the definition of a new type of Node (header)
 */
/**
 * \class bbtk::NodePatern 
 * \brief Pattern for the definition of a new type of Node 
 */


#ifdef _USE_VTK_


#ifndef __bbvtkMIPCreator_h_INCLUDED__
#define __bbvtkMIPCreator_h_INCLUDED__


#include "bbtkAtomicBlackBox.h"

#include "vtkImageData.h"
#include <vtkImageShiftScale.h>
#include <vtkVolumeRayCastMIPFunction.h>
#include <vtkVolumeRayCastMapper.h>
#include "vtkProp3D.h"
#include "vtkVolume.h"

#include "bbvtk_EXPORT.h"

namespace bbvtk
{


  
  class bbvtk_EXPORT MIPCreator : public bbtk::AtomicBlackBox
  { 
  public:
    BBTK_BLACK_BOX_INTERFACE(MIPCreator,bbtk::AtomicBlackBox);
    //    BBTK_DECLARE_INPUT(Contour,int);
    BBTK_DECLARE_INPUT(In,vtkImageData *);
    BBTK_DECLARE_INPUT(Shift,int);
    BBTK_DECLARE_INPUT(Scale,float);
    BBTK_DECLARE_OUTPUT(Out,vtkProp3D*);
    BBTK_PROCESS(Process);
    void Process();
    
  protected:
  private:
    vtkImageShiftScale				*mCast;
    vtkVolumeRayCastMIPFunction		*mMIP;
    vtkVolumeRayCastMapper			*mMapper;
    vtkVolume						*mVolume;
  };
   
  //=================================================================
  // UserBlackBox description
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(MIPCreator,bbtk::AtomicBlackBox);
  BBTK_NAME("MIPCreator");
  BBTK_AUTHOR("laurent.guigues@creatis.insa-lyon.fr");
  BBTK_DESCRIPTION("Creates a Maximum Intensity Projection (MIP) view of a 3D image. Creates a vtkProp3D object to insert into a 3D scene (e.g. a Viewer3D)");
  BBTK_CATEGORY("3D object creator");
  BBTK_INPUT(MIPCreator,Shift,"Gray scale shift",int,"");
  BBTK_INPUT(MIPCreator,Scale,"Gray scale scaling",float,"");
  BBTK_INPUT(MIPCreator,In,"Input Image",vtkImageData *,"");
  BBTK_OUTPUT(MIPCreator,Out,"MIP object to plug into a 3D viewer",vtkProp3D*,"");
  BBTK_END_DESCRIBE_BLACK_BOX(MIPCreator);
  //=================================================================


}//namespace bbvtk

#endif  //__bbtvtkMIPCreator_h__

#endif //_USE_VTK_

