/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbwxLayoutSplit.h,v $
  Language:  C++
  Date:      $Date: 2009/05/15 14:58:03 $
  Version:   $Revision: 1.9 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 */
/**
 * \file 
 * \brief Widget which splits a window into two parts 
 */
/**
 * \class bbwx::LayoutSplit
 * \brief Widget which splits a window into two parts 
 */


#ifdef USE_WXWIDGETS


#ifndef __bbWxLayoutSplit_h__
#define __bbWxLayoutSplit_h__

#include "bbtkWxBlackBox.h"
#include <wx/splitter.h>

#include "bbwx_EXPORT.h"

namespace bbwx
{
  
  
  
  
  //=================================================================
  class bbwx_EXPORT LayoutSplit : public bbtk::WxBlackBox
  {
    
    BBTK_BLACK_BOX_INTERFACE(LayoutSplit,bbtk::WxBlackBox);
    BBTK_DECLARE_INPUT(Widget1,wxWindow*);
    BBTK_DECLARE_INPUT(Widget2,wxWindow*);
    BBTK_DECLARE_INPUT(Orientation,std::string);
    BBTK_DECLARE_INPUT(Proportion,int);
    BBTK_PROCESS(Process);
    BBTK_CREATE_WIDGET(CreateWidget);
    BBTK_ON_SHOW_WIDGET(OnShowWidget);
    
    void Process();
    void CreateWidget(wxWindow*);
    void OnShowWidget();

  protected:

  };
  //=================================================================
  
 
  //=================================================================
  // BlackBox description
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(LayoutSplit,bbtk::WxBlackBox);
  BBTK_NAME("LayoutSplit");
  BBTK_AUTHOR("laurent.guigues@creatis.insa-lyon.fr");
  BBTK_DESCRIPTION("Widget which splits a window in two fixed size parts (wxSplitterWindow)");
  // Already inserted for any WxBlackBox  BBTK_CATEGORY("widget");
  BBTK_INPUT(LayoutSplit,Widget1,"Upper or left widget",wxWindow*,"");
  BBTK_INPUT(LayoutSplit,Widget2,"Lower or right widget",wxWindow*,"");
  BBTK_INPUT(LayoutSplit,Orientation,"Orientation (default H), 0=H=HORIZONTAL , 1=V=VERTICAL",std::string,"");
  BBTK_INPUT(LayoutSplit,Proportion,"Proportion (in percent) of the first children in the window",int,"");
  BBTK_END_DESCRIBE_BLACK_BOX(LayoutSplit);
  //=================================================================
}

//namespace bbtk
#endif  //__bbtkWxLayoutSplit_h__

#endif //USE_WXWIDGETS
